using System.Collections.Generic;
using UnityEngine;

namespace game
{
    /// <summary>
    /// La classe <c>EventInfo</c> permet de crer un vnement avec toutes ses informations ncessaires.
    /// Les vnments sont composs d'un code unique (leur nom), d'une description, d'une dure (combien de temps ils vont durer aprs leur apparition), de 2 multiplieurs (le premier pour rajouter
    /// un simple multiplieur au prix, par exemple *0.9 ou 1.1, multiplieur fixe qui reste actif tant que l'vent l'est
    /// et le deuxime est un multiplieur qui atteint son pic de multiplication au milieu de sa dure, ex : il vaut 2, et l'vnement dure 5, alors il oscilera un peu prs comme a : 1.33, 1.66, 2, 1.66, 1.33),
    /// ensuite il y a 3 boolens (le premier pour savoir si l'vnement atteint les plantes, le deuxime si il atteint les graines et le troisime si il atteint les outils.
    /// Ensuite il y a la liste des plantes (ou des graines, a dpend ce que a atteint) et la liste des outils, pour les outils atteint.
    /// Les derniers paramtres sont : la probabilit d'apparition de l'vnement, un vnement ne peut qu'arriver  partir du xime jour,
    /// l'image de l'vnement et le dernier paramtre correspond au temps  partir duquel l'vnement peut rapparatre aprs que celui-ci est commenc (ex : un event  une dure de 5, un cooldown de 10,
    /// et un unlockAfter de 0, il peut arriver ds le 1er jour, et 10 jours aprs qu'il soit arriv, il peut rapparatre).
    /// Elle contient 2 constructeurs, un pour crer les vnements et un en cas d'erreur.
    /// De plus, elle contient 5 mthodes, pour obtenir son nom, sa dure, sa description et ce que l'vnement affecte.
    /// </summary>
    public class EventInfo
    {
        public string namee;
        public string description;
        public int length;
        public double mutliplierBase;
        public double mutliplierProg;
        public bool targetPlant;
        public bool targetSeed;
        public bool targetTool;
        public List<EnumTypePlant> targetsPlant = new List<EnumTypePlant>();
        public List<string> targetsTool = new List<string>();
        public int probability;
        public int unlockableAfter;
        public Sprite imageLink;
        public int cooldown;

        /// <summary>
        /// Ce premier constructeur permet de crer un vnement.
        /// </summary>
        /// <param name="namee">le nom de l'vnement</param>
        /// <param name="description">la description de l'vnement</param>
        /// <param name="lenght">la dure de l'vnement</param>
        /// <param name="mutliplier">simple multiplieur au prix, multiplieur fixe qui reste actif tant que l'vnement l'est</param>
        /// <param name="mutliplierProg">multiplieur qui atteint son pic de multiplication au milieu de sa dure</param>
        /// <param name="targetPlant">savoir si l'vnement atteint les plantes</param>
        /// <param name="targetSeed">savoir si l'vnement atteint les graines</param>
        /// <param name="targetTool">savoir si l'vnement atteint les outils</param>
        /// <param name="targetsPlant">liste des plantes (ou des graines) atteintes</param>
        /// <param name="targetsTool">liste des outils atteints</param>
        /// <param name="probability">probabilit d'apparition de l'vnement</param>
        /// <param name="unlockableAfter">un vnements ne peut qu'arriver  partir du xime jour</param>
        /// <param name="imageLink">le lien de l'image de l'vnement</param>
        /// <param name="cooldown">temps  partir duquel l'vnement peut rapparatre aprs que celui-ci ai commenc</param>
        public EventInfo(string namee,
            string description,
            int lenght,
            double mutliplier,
            double mutliplierProg,
            bool targetPlant,
            bool targetSeed,
            bool targetTool,
            List<EnumTypePlant> targetsPlant,
            List<string> targetsTool,
            int probability,
            int unlockableAfter,
            Sprite imageLink,
            int cooldown)
        {
            this.namee = namee;
            this.description = description;
            this.length = lenght;
            this.mutliplierBase = mutliplier;
            this.mutliplierProg = mutliplierProg;
            this.targetPlant = targetPlant;
            this.targetSeed = targetSeed;
            this.targetTool = targetTool;
            this.targetsPlant = targetsPlant;
            this.targetsTool = targetsTool;
            this.probability = probability;
            this.unlockableAfter = unlockableAfter;
            this.imageLink = imageLink;
            this.cooldown = cooldown;
        }

        /// <summary>
        /// Ce deuxime constructeur permet de crer un vnement "Error" en cas d'erreur.
        /// </summary>
        public EventInfo()
        {
            this.namee = "Error";
            this.description = "Error, using an empty constructor";
            this.length = -1;
            this.mutliplierBase = -1;
            this.mutliplierProg = 0;
            this.targetPlant = false;
            this.targetSeed = false;
            this.targetTool = false;
            this.targetsPlant = new List<EnumTypePlant>();
            this.targetsTool = new List<string>();
            this.probability = -1;
            this.unlockableAfter = -1;
            this.imageLink = Game.getDefaultSprite();
            this.cooldown = -1;
        }

        /// <summary>
        /// La mthode <c>getName</c> permet d'obtenir le nom de l'vnement.
        /// </summary>
        /// <returns>Elle retourne son nom</returns>
        public string getName()
        {
            return this.namee;
        }

        /// <summary>
        /// La mthode <c>getLength</c> permet d'obtenir la dure de l'vnement.
        /// </summary>
        /// <returns>Elle retourne sa dure</returns>
        public int getLength()
        {
            return this.length;
        }

        /// <summary>
        /// La mthode <c>getDescription</c> permet d'obtenir la description de l'vnement.
        /// </summary>
        /// <returns>Elle retourne sa description</returns>
        public string getDescription()
        {
            return this.description;
        }

        /// <summary>
        /// La mthode <c>getTarget</c> permet d'obtenir la liste des plantes ou des graines que l'vnement affecte.
        /// </summary>
        /// <returns>Elle retourne la chane de caractres des plantes ou des graines, spares par des virgules</returns>
        public string getTarget()
        {
            string rtr = string.Empty;
            for (int i = 0; i < targetsPlant.Count; i++)
            {
                rtr += targetsPlant[i] + ", ";
            }
            if(rtr.Length <= 0)
            {
                return rtr;
            }
            rtr = rtr.Substring(0, rtr.Length - 2);
            return rtr;
        }

        /// <summary>
        /// La mthode <c>ifTarget</c> permet de savoir ce que l'vnement atteint.
        /// </summary>
        /// <returns>Une chaine de caractres qui dit ce que a atteint</returns>
        public string ifTarget()
        {
            if (this.targetSeed == true)
            {
                return "Les graines de : ";
            }
            else if (this.targetPlant == true)
            {
                return "Les plantes de : ";
            }
            else
            {
                return "Les outils : ";
            }
        }
    }
}